from typing import Dict, List


class KContext:
    def __init__(self) -> None:
        self.symbols: Dict[str, int] = {}
        self._symbol_str: List[str] = []
        self._symbol_bytes: List[bytes] = []

    def ss(self, s: str) -> int:
        idx = self.symbols.setdefault(s, len(self.symbols))
        if idx == len(self._symbol_str):
            bs = bytes(s, "utf-8") + b"\x00"
            self._symbol_bytes.append(bs)
            self._symbol_str.append(s)
        return idx

    def lookup_str(self, idx: int) -> str:
        return self._symbol_str[idx]

    def lookup_bytes(self, idx: int) -> bytes:
        return self._symbol_bytes[idx]



from aiokdb.context import KContext


def test_context() -> None:
    kcon = KContext()
    assert kcon.ss("hello") == 0
    assert kcon.ss("world") == 1
    assert kcon.ss("hello") == 0
    assert kcon.ss("world") == 1
    assert kcon.ss("how") == 2
    assert kcon.ss("💩") == 3
    assert len(kcon.symbols) == 4

    assert kcon.lookup_str(2) == "how"
    assert kcon.lookup_bytes(2) == b"how\00"



import asyncio

import pytest

from aiokdb import KObj
from aiokdb.server import ConnectionClosed, KdbReader, KdbWriter


class DummyWriter(asyncio.StreamWriter):
    def __init__(self) -> None:
        pass

    def close(self) -> None:
        self.closed = True

    def is_closing(self) -> bool:
        return True

    async def wait_closed(self) -> None:
        pass


class DummyReader(KdbReader):
    def __init__(self) -> None:
        pass


@pytest.mark.asyncio
async def test_kdbwriter_close_sets_exception_on_pending_futures() -> None:
    writer = KdbWriter(DummyWriter(), DummyReader())
    fut1: asyncio.Future[KObj] = asyncio.Future()
    fut2: asyncio.Future[KObj] = asyncio.Future()
    writer._completions.append(fut1)
    writer._completions.append(fut2)
    writer.close()
    # Both futures should now be done and have ConnectionClosed exception
    for fut in (fut1, fut2):
        assert fut.done()
        with pytest.raises(ConnectionClosed):
            fut.result()
